import "./Question.ts"; // contains schema

const db = getDb("questions.sqlite");

export interface PendingQuestionData {
  date: string;
  prompt: string;
  sourceName: string | "unknown";
  sourceLocation: string | "unknown";
  sourceVPN: string | null;
}

/** PendingQuestion is actually a `Question` but with type == .pending */
export class PendingQuestion {
  /**
   * questions are identified by a 'minute identifier'
   * which is time / 60000 (minutes since epoch)
   */
  qmid!: number;
  private type!: QuestionType.pending;
  private text!: string;
  private cached_json?: PendingQuestionData;

  // -- instance ops --

  /** "https://paperclover.net/q+a/${id}" - #YYMMDDHHMM */
  get id(): string {
    return formatQuestionId(this.date);
  }
  get date(): Date {
    return new Date(this.qmid * 1000 * 60);
  }
  get data(): PendingQuestionData {
    assert(this.type === QuestionType.pending);
    return this.cached_json ??= JSON.parse(this.text);
  }
  get prompt() {
    return this.data.prompt;
  }
  get sourceName() {
    return this.data.sourceName;
  }
  get sourceLocation() {
    return this.data.sourceLocation;
  }
  get sourceVPN() {
    return this.data.sourceVPN;
  }

  // -- static ops --

  static getAll() {
    return getAllQuery.iter();
  }
  static getByDate(timestamp: Date) {
    const qmid = Math.floor(timestamp.getTime() / 1000 / 60);
    return getByDateQuery.get(qmid);
  }
}

const getAllQuery = db.prepare(`
  SELECT * FROM questions WHERE type = ${QuestionType.pending} ORDER BY qmid DESC
`).as(PendingQuestion);

const getByDateQuery = db.prepare<[qmid: number]>(`
  SELECT * FROM questions WHERE qmid = ? AND type = ${QuestionType.pending} LIMIT 1
`).as(PendingQuestion);

import { getDb } from "#sqlite";
import assert from "node:assert";
import { QuestionType } from "./Question.ts";
import { formatQuestionId } from "../format.ts";
